"""
Module for animating time-dependent PDE solutions.

example usage:

import matplotlib.pyplot as plt
from numpy import linspace, cos, sin, pi
import AnimatePlot as ap

N = 100
x = linspace(0,1,N)
t, dt, T = 0, 0.01, 2.0

def makePlot(): 
 	plt.plot(x, u, 'g')
 	plt.ylim([-1.5,1.5])
 
while t<T:
    u = sin(pi*x)*cos(pi*t) + 0.5*sin(3*pi*x)*cos(3*pi*t)
    t += dt
    ap.animate(makePlot)
    
"""


import sys
import matplotlib.pyplot as plt


def animate(draw_fig, *args, **kwargs):
    plt.clf()    
    draw_fig(*args, **kwargs)
    plt.draw_all()
    plt.pause(1e-4)              # allow time to draw
    figures = plt.get_fignums()  # exit when figure is closed
    if not figures: sys.exit()
    


def figure(*args, **kwargs):
    plt.ion()
    plt.figure(*args, **kwargs)
