"""
Compares the interpolation of a 1d function using a constant shape parameter
and a variable shape parameter.

run variableShapeInterp1d.py

"""

from numpy import linspace, inf, ones, zeros
from numpy.linalg import cond, norm
import rbfCenters as rc
from gax import *

PROBLEM = 1

phi = gax() 
safe = True
mu = 0

N, M = 44, 175
xc = rc.xGL(N)           # centers
x = linspace(-1,1,M)     # evaluation points

if PROBLEM == 1:    # infinitely differentiable function
	from F1a import *
	fn = F1a()
	f, fe, fp = fn.F(xc), fn.F(x), fn.x1(xc)
else:               # constant function
	f, fe, fp = ones((N,)), ones((M,)), zeros((N,))

r, re = phi.distanceMatrix1d(xc), phi.distanceMatrix1d(xc,x)		
	
# ---------- constant shape, interpolation -----------------------------

s = 1.0
B = phi.rbf(r,s)
kappa = cond(B)
a = phi.solve(B,f,mu,safe)
H = phi.rbf(re,s)
fa = H@a
er = norm(fa - fe, inf)

# ---------- variable shape, interpolation -----------------------------

sMin, sMax = 0.5, 1.5
opt = 3
sn, sm = phi.variableShape(sMin,sMax,opt,N,M)

Bv = phi.rbf(r,sn)
kappaV = cond(Bv)
av =  phi.solve(Bv,f,mu,safe)
Hv = phi.rbf(re,sm)
fav = Hv@av
erv = norm(fav - fe, inf)

# ---------- constant shape, differentiation ---------------------------

H = phi.D1(r,s,r)
fa = H@a
erd = norm(fa - fp, inf)

# ---------- variable shape, differentiation ---------------------------

Hv = phi.D1(r,sn,r)
fav = Hv@av
erdv = norm(fav - fp, inf)

# ----------------------------------------------------------------------

print('variable shape system matrix condition number {:1.2e}'.format(kappaV))
print('constant shape system matrix condition number {:1.2e}'.format(kappa))
print(' ')
print('variable shape interpolation error {:1.10e}'.format(erv))
print('constant shape interpolation error {:1.10e}'.format(er))
print(' ')
print('variable shape derivative error {:1.2e}'.format(erdv))
print('constant shape derivative error {:1.2e}'.format(erd))
