"""
Tests accuracy of rational RBF differentiation of a smooth 1d function.

reference: S. A. Sarra and Y. Bai. A rational radial basis function method for accurately 
           resolving discontinuities and steep gradients. Applied Numerical Mathematics 130, 
           p. 131-142, 2018.  DOI: 10.1016/j.apnum.2018.04.001

usage: run diffRRBF.py

"""

from numpy import linspace, pi, sin, cos, inf, exp
from numpy.linalg import norm
from iqx import *
import rbfRational as rat
import rbfCenters as rc
import matplotlib.pyplot as plt

plt.close('all')

N = 80
#xc = linspace(-1,1,N)
xc = rc.xKT(N,-1,1,0.99)

phi = iqx()
s = 4.0
mu = 0

f, fp, fpp = sin(pi*xc), pi*cos(pi*xc), -pi**2*sin(pi*xc)
#f, fp, fpp = xc**3, 3*xc**2, 6*xc
#f, fp, fpp = exp(xc), exp(xc), exp(xc)

fa1, fa2 = rat.diff1dOne(phi,s,f,xc,mu)

print('ratrional 1st derivative error: {:1.2e}'.format(norm(fa1 - fp, inf)))
print('ratrional 2nd derivative error: {:1.2e}'.format(norm(fa2 - fpp, inf)))
print(' ')

plt.plot(xc,fa1,'b',xc,fa2,'g')
plt.plot(xc,fp,'k--',xc,fpp,'k--')
plt.show()
