"""
  Uses global RBF differentiation to approximate L1 = u_x + u_y and
  L2 = u_{xx} + u_{yy} on a cicular domain

  The NSPD method is used to select the shape parameter and MDI 
  regularization is used in form the differentation matrices.
  
  usage:  run differentiationGlobal.py
  
"""

from numpy.linalg import norm, cond
from numpy import inf
import rbfCenters as rc
from rbfClassX import rbfx
from iqx import *
import warnings                     # ignore warnings of ill-conditioning
warnings.filterwarnings('ignore')

N = 1500              # number of centers in a rectangle then restricted to a cirlce
xc, yc = rc.circleCenters(N,ch=3,center=(0.5,0.5))
N = len(xc)            # recalculate N as it will be less than the original N 
X = list( zip(xc,yc) )

from F2b import *   # derivative of this function being approximated
fn = F2b()
f = fn.F(xc,yc)
fDiv, fLap = fn.div(xc,yc), fn.L(xc,yc)  # exact derivatives

phi = iqx()     # inverse quadratic RBF
s0 = 2          # initial shape guess
s, _ = rbfx.spdShape(phi, X, s0, ds=0.02, mu=0.0, verbose=False)
r, rx, ry = rbfx.distanceMatrix(X, x=None, nargout = 3)
B = phi.rbf(r,s)                 # system matrix

mu, safe = 5e-15, False    # MDI regularization parameter, False => Cholesky

Hd, Hl = phi.div(r,s,rx,ry), phi.L(r,s)              # derivative evaluation matrices
Dv, Dl = phi.dm(B,Hd,mu,safe), phi.dm(B,Hl,mu,safe)  # differentiation matrices
fd, fl = Dv@f, Dl@f

print('        GLOBAL, N = {:d}'.format(N))
print('                      L1 error = {:4.2e}'.format( norm(fd - fDiv, inf )))
print('                      L2 error = {:4.2e}'.format( norm(fl - fLap, inf )))
print('       shape via SPD selection = {:1.2f}'.format(s))
print('system matrix condition number = {:4.2e}'.format( cond(B) ))



