"""
Uses local RBF differentiation to approximate L1 = u_x + u_y and
L2 = u_{xx} + u_{yy} on a cicular domain

The NSPD method is used to select the shape parameter on each stencil
and MDI regularization is used in solving the linear system for the
weights.

usage: run differentiationLocal.py

"""

from numpy.linalg import norm
from numpy import inf
import rbfCenters as rc
from rbfClassX import rbfx
from iqx import *
import warnings                     # ignore warnings of ill-conditioning
warnings.filterwarnings('ignore')

N = 1500                
xc, yc = rc.circleCenters(N,ch=3,center=(0.5,0.5))
N, X = len(xc), list( zip(xc,yc) )

from F2b import *   # derivative of this function being approximated
fn = F2b()
f = fn.F(xc,yc)
fDiv, fLap = fn.div(xc,yc), fn.L(xc,yc)  # exact derivatives

phi = iqx()         # inverse quadratic

import rbfLocal as LR 

def L1(r,x,y,c,phi): return phi.div(r,c,x,y)    # divergence
def L2(r,x,y,c,phi): return phi.L(r,c)          # Laplacian

ns = 80                   # local stencil size
st = LR.stencils(X,ns)    # stencils via a kd-tree method      
shape = LR.shapes(X,st,phi,mu=0,s0=0.6,ds=0.02)   # SPD shape parameter selection

mu = 5e-15   # MDI regularization
DLv, DLl = LR.weights(X,st,phi,shape,L1,mu), LR.weights(X,st,phi,shape,L2,mu)
fdl, fll = DLv@f, DLl@f        # sparse matrix mult

print('local stencil size: {:d}'.format(ns))
print('         L1 error = {:4.2e}'.format( norm(fdl - fDiv, inf )))
print('         L2 error = {:4.2e}'.format( norm(fll - fLap, inf )))
print(' ')
