"""
Compares the accuracy of global and local RBF differentiation.

usage: run globalVsLocalFixedN.py

"""
 
import matplotlib.pyplot as plt
from numpy import zeros, linspace, sin, cos, arange, array, pi, roll, inf
from numpy.linalg import norm
import rbfLocal as LR 
from rbfClassX import rbfx
from iqx import *

import warnings
warnings.filterwarnings('ignore')
plt.close('all')

from Franke import *
fn = Franke()

# NOTE: This is not part of the PRBFT.  It is available from:
#   https://github.com/treverhines/RBF
from rbf.pde.nodes import min_energy_nodes
# from rbf.nodes import menodes

phi = iqx()
safe = False  # use Cholesky rather than LU

n = arange(10,201,10,dtype=int)   # local stencil sizes
N = 5000                          # fixed number of centers
P = len(n)
er = zeros(P)

mu = 5e-15   # MDI regularization parameter
s0L = 1      # local initial shape
ds = 0.01    # increment for shape parameter search
muS = 0      # regularization parameter for shape selection


def L(r,x,y,c,phi): return phi.div(r,c,x,y)

# --- domain boundary and center setup ---------------------------------

M = 200
t = linspace(0,2*pi,M + 1)
t = t[:-1]
radius = 0.45*(0.1*sin(10*t) + 1.0)
vert = array([0.5+radius*cos(t),0.5+radius*sin(t)]).T
smp = array([arange(M),roll(arange(M),-1)]).T

# centers, smpid = menodes(N,vert,smp) 
# xc, yc = centers[:,0], centers[:,1]

centers = min_energy_nodes(N,(vert,smp)) 
xc, yc = centers[0][:,0], centers[0][:,1]

f, fe = fn.F(xc,yc), fn.div(xc,yc)
X = list( zip(xc,yc) )

# -------------- local RBF ---------------------------------------------

for i in range(0,P):
	print('n = {:d}'.format(n[i]))
	ns = n[i]
	st = LR.stencils(X,ns) 
	sL = LR.shapes(X,st,phi,muS,s0L,ds)
	s0L = sL[int(ns/2)]
	D = LR.weights(X,st,phi,sL,L,mu)
	faL = D@f
	er[i] = norm(faL - fe,inf)
	print('local error: {:1.2e}'.format(er[i]))
	print(' ')
    
# ----------------------------------------------------------------------

plt.semilogy(n,er,'b')
plt.xlabel('n'), plt.ylabel('|error|')
plt.show()
