"""
Compares the accuracy of global and local RBF derivative approximation.

usage: run globalVsLocalVaryN.py

"""

import matplotlib.pyplot as plt
from numpy import zeros, arange, linspace, sin, cos, roll, inf, pi, array
from numpy.linalg import norm
import rbfLocal as LR 
from rbfClassX import rbfx
from iqx import *

import warnings
warnings.filterwarnings('ignore')
plt.close('all')

from Franke import *
fn = Franke()

# NOTE: This is not part of the PRBFT.  It is available from:
#   https://github.com/treverhines/RBF
from rbf.pde.nodes import min_energy_nodes
# from rbf.nodes import menodes

phi = iqx()
safe = False    # use Cholesky rather than LU

N = arange(1000,5001,500,dtype=int)   # number of centers
P = len(N)
er, erL = zeros(P), zeros(P)

ns = 40           # fixed center size
print(' ')
print('local stencil size: {:d}'.format(ns))
print(' ')
mu = 5e-15   # MDI regularization parameter
s0L = 1      # local initial shape
ds = 0.01    # increment for shape parameter search
muS = 0      # regularization parameter for shape selection
s0G = 2      # global initial shape

def L(r,x,y,c,phi): return phi.div(r,c,x,y)  

# --- domain boundary and center setup ---------------------

M = 200
t = linspace(0,2*pi,M + 1)
t = t[:-1]
radius = 0.45*(0.1*sin(10*t) + 1.0)
vert = array([0.5+radius*cos(t),0.5+radius*sin(t)]).T
smp = array([arange(M),roll(arange(M),-1)]).T

# ----------------------------------------------------------


for i in range(0,P):
	print('N = {:d}'.format(N[i]))
	
	# centers, smpid = menodes(N[i],vert,smp) 
	# xc, yc = centers[:,0], centers[:,1]
	
	centers = min_energy_nodes(N[i],(vert,smp)) 
	xc, yc = centers[0][:,0], centers[0][:,1]
	
	
	f, fe = fn.F(xc,yc), fn.div(xc,yc)
	
# -------------- local RBF ---------------------------------
	
	X = list( zip(xc,yc) )
	st = LR.stencils(X,ns) 
	sL = LR.shapes(X,st,phi,muS,s0L,ds)
	s0L = sL[int(N[i]/2)]
	D = LR.weights(X,st,phi,sL,L,mu)
	faL = D@f
	erL[i] = norm(faL - fe,inf)
	print('local error: {:1.2e}'.format(erL[i]))
    
# ------------ global RBF ----------------------------------

	r, rx, ry = phi.distanceMatrix2d(xc,yc)
	sg, _ = rbfx.spdShape(phi,X,s0G,ds,muS)
	s0G = sg
	B = phi.rbf(r,sg)
	a = phi.solve(B,f,mu,safe)
	H = phi.div(r,sg,rx,ry)
	fa = H@a
	er[i] = norm(fa - fe,inf)
	print('global error: {:1.2e}'.format(er[i]))
	print(' ')
	
# ----------------------------------------------------------------------
   
l1, l2 = plt.semilogy(N,er,'g--',N,erL,'b')
plt.legend((l1, l2), ('global', 'local'))
plt.xlabel('N'), plt.ylabel('|error|')
plt.show()
