"""
Examines eigenvalue (ew) stability of a globa RBF method for the 2d diffusion 
equation with zero Dirichlet eigenvalues on a circular domain.

Three options are available for improving ew stability: MDI regularization,
a inverse system matrix filter, and Laplacian power artifical viscosity.  One
or more approach may be used at the same time.

usage: run heatCircleGlobalEws.py

note: max real ew: 0.0 with only MDI
          
"""

import matplotlib.pyplot as plt
import rbfCentersLib as cl
import rbfMisc as rm
import rbfCenters as rc
from numpy import arange
from numpy.linalg import cond

import warnings
warnings.filterwarnings('ignore')
plt.close('all')

from rbfClassX import rbfx

from gax import *
phi = gax()

N, Nb = 6000, 350
bpi = arange(Nb)
xi, yi = rc.circleCenters(N,cluster=False,ch=3,R=1,plt=False)
x, y = rc.circleBndCenters(xi,yi,1,Nb,tol=0.0125,plt=False)
x += 1
y += 1

print('boundary centers: {:d}'.format(Nb))
print('   total centers: {:d}'.format(len(x)))

shape = 13.5   # SPD shape selection
r, rx, ry = phi.distanceMatrix2d(x, y)
B = phi.rbf(r,shape)
print('K(B): {:1.2e}'.format(cond(B,2)))

mu, safe = 5e-15, False
H = phi.L(r,shape)
D2 = phi.dm(B, H, mu, safe)
D2[bpi,:] = 0

rm.dmEWs(D2,dt=0.0001,p1=True,p2=True,verbose1=True,verbose2=True)
