"""
Demonstrates MDI regularization and NSPD shape parameter selection.

usage: run mdiRegularization.py

"""

import matplotlib.pyplot as plt
from numpy import arange, zeros, inf, eye, linspace, nonzero, ravel
from numpy.linalg import norm, cond
from iqx import *
import rbfCenters as rc
from F1a import *
import rbfCenters as rc

import warnings
warnings.filterwarnings('ignore')
plt.close('all') 

phi = iqx()        # inverse quadratic RBF

N, M = 60, 200     # centers and evaluation points
mu = 5e-15         # MDI regularization parameter

xc, x = rc.xKT(N,-1,1,0.98), linspace(-1,1,M)              # centers, evaluation points
r, re = phi.distanceMatrix(xc), phi.distanceMatrix(xc, x)

fn = F1a()                   # function to interpolate
f, fe = fn.F(xc), fn.F(x)   

s = arange(0.8,4.01,0.05)    # range of shape parameters
P = len(s)
er, kappa, method = zeros(P), zeros(P), zeros(P)
erb, kappab  = zeros(P), zeros(P)

for k in range(P):
	B = phi.rbf(r,s[k])                   # system matrix
	kappa[k] = cond(B)
	a, method[k] = phi.solveAlt(B,f,0)    # Cholesky or LU without MDI
	H = phi.rbf(re,s[k])
	fa = H@a
	er[k] = norm(fa - fe,inf)
	
	kappab[k] = cond(B + mu*eye(N))
	a  = phi.solve(B,f,mu,safe=False)    # Cholesky with MDI regularization
	fab = H@a
	erb[k] = norm(fab - fe,inf)
    
I, = nonzero(ravel(method<1))
I0 = I[0] 

print('Cholesky fails at s = {:1.2f}'.format(s[I0]))

l1, l2, l3 = plt.semilogy(s,er,'b',s[I0],er[I0],'ko',s,erb,'g--')
plt.legend((l1, l3), ('no MDI', 'with MDI'))
plt.xlabel('shape parameter'), plt.ylabel('|error|')

plt.figure()
l1, l2, l3 = plt.semilogy(s,kappa,'b',s[I0],kappa[I0],'ko',s,kappab,'g--')
plt.xlabel('shape parameter'), plt.ylabel('condition number')
plt.legend((l1, l3), ('no MDI', 'with MDI'))
plt.show()

