"""
Compares the execution time of the standard and centrosymmetric algorithms
for calculating the 2-norm condition number of a centrosymmetric matrix.

reference: S. A. Sarra. Radial Basis Function Methods - the case of symmetric domains.  
    Numerical Methods for Partial Differential Equations, 2018. DOI:10.1002/num.22272 

usage: run centroCondBench.py

"""

from numpy import arange, zeros, linspace
from numpy.random import rand
from numpy.linalg import cond
from iqx import *
import rbfCentro as cn
import time
import matplotlib.pyplot as plt
plt.close('all')

phi = iqx()
s = 230
mu = 0
its = 5
Nv = arange(600,4101,200)
Ns = len(Nv)
cTime, ncTime = zeros((Ns,)), zeros((Ns,))

for k in range(Ns):
	N = Nv[k]
	Nh = int(N/2)
	x = linspace(-100,100,N)
	f = rand(N,1)
	startTime = time.time()
	for j in range(its):
		r = phi.distanceMatrix1d(x[:Nh],x)     # construct half-sized distance matrix
		B = phi.rbf(r,s)                       # half-sized system matrix
		kappaB, kappaL, kappaM = cn.centroConditionNumber(B,mu)
	cTime[k] = time.time() - startTime 
	
for k in range(Ns):
	N = Nv[k]
	x = linspace(-100,100,N)
	f = rand(N,1)
	startTime = time.time()
	for j in range(its):
		r = phi.distanceMatrix1d(x)     
		B = phi.rbf(r,s)                       
		kappa = cond(B)
	ncTime[k] = time.time() - startTime 
	

plt.plot(Nv,cTime/ncTime,'b*') #,Nv,ones((Ns,)),'g--')
plt.xlabel('N'), plt.ylabel('execution time ratio')
plt.show()

		
	
