"""
Compares the execution times of centro versus standard algorithms
for constructing a centro derivative matrix.

reference: S. A. Sarra. Radial Basis Function Methods - the case of symmetric domains.  
    Numerical Methods for Partial Differential Equations, 2018. DOI:10.1002/num.22272 

usage: run centroDmFormBench.py

"""

from numpy import arange, zeros, linspace
from iqx import *
import rbfCentro as cn
import time
import matplotlib.pyplot as plt
plt.close('all')

phi = iqx()
s = 230
safe = False
rho = 1
mu = 0
its = 5
Nv = arange(350,8201,200)
Ns = len(Nv)
cTime, ncTime = zeros((Ns,)), zeros((Ns,))

for k in range(Ns):
	N = Nv[k]
	print(N)
	Nh = int(N/2)
	x = linspace(-100,100,N)
	startTime = time.time()
	for j in range(its):
		r = phi.distanceMatrix1d(x[:Nh],x)     # construct half-sized distance matrix
		B = phi.rbf(r,s)                       # half-sized system matrix
		H = phi.D1(r,s,r)
		D = cn.centroDM(B,H,N,rho,mu,safe)
	cTime[k] = time.time() - startTime 
	
for k in range(Ns):
	N = Nv[k]
	print(N)
	Nh = int(N/2)
	x = linspace(-100,100,N)
	startTime = time.time()
	for j in range(its):
		r = phi.distanceMatrix1d(x)     
		B = phi.rbf(r,s)
		H = phi.D1(r,s,r)
		D = phi.dm(B,H,mu,safe)
	ncTime[k] = time.time() - startTime 
	

plt.plot(Nv,cTime/ncTime,'b*') #,Nv,ones((Ns,)),'g--')
plt.xlabel('N'), plt.ylabel('execution time ratio')
plt.show()

		
	
