"""
Compares the execution time of centro versus the standard algorithm for
matrix-vector multiplication.

reference: S. A. Sarra. Radial Basis Function Methods - the case of symmetric domains.  
Numerical Methods for Partial Differential Equations, 2018. DOI:10.1002/num.22272

usage: run centroMultBench.py

"""

from numpy import arange, zeros, linspace
from numpy.random import rand
from iqx import *
import rbfCentro as cn
import time
import matplotlib.pyplot as plt
plt.close('all')

phi = iqx()
s = 230
safe = True
rho = 1
mu = 0
its = 1
Nv = arange(600,3001,100)
Ns = len(Nv)
cTime, ncTime = zeros((Ns,)), zeros((Ns,))

for k in range(Ns):
	N = Nv[k]
	print(N)
	Nh = int(N/2)
	x = linspace(-100,100,N)
	u = rand(N)
	r = phi.distanceMatrix1d(x[:Nh],x)     # construct half-sized distance matrix
	B = phi.rbf(r,s)                       # half-sized system matrix
	H = phi.D1(r,s,r)
	D = cn.centroDM(B,H,N,rho,mu,safe)
	L,M = cn.centroDecomposeMatrix(D,rho)
	startTime = time.time()
	for j in range(its):
		uac = cn.centroMult(u,L,M,rho)
	cTime[k] = time.time() - startTime 
	
for k in range(Ns):
	N = Nv[k]
	print(N)
	x = linspace(-100,100,N)
	u = rand(N)
	r = phi.distanceMatrix1d(x)     
	B = phi.rbf(r,s)
	H = phi.D1(r,s,r)
	D = phi.dm(B,H,mu,safe)
	startTime = time.time()
	for j in range(its):
		ua = D@u
	ncTime[k] = time.time() - startTime 
	

plt.plot(Nv,cTime/ncTime,'b*') #,Nv,ones((Ns,)),'g--')
plt.xlabel('N'), plt.ylabel('execution time ratio')
plt.show()

		
	
