"""
Compares the execution time of the standard and centrosymmetric algorithms
for solving a centrosymmetric linear system

reference: S. A. Sarra. Radial Basis Function Methods - the case of symmetric domains.  
Numerical Methods for Partial Differential Equations, 2018. DOI:10.1002/num.22272

usage: run centroSystemBench.py

"""

from numpy import arange, zeros, linspace
from iqx import *
import rbfCentro as cn
import time
from numpy.random import rand
import matplotlib.pyplot as plt
plt.close('all')

phi = iqx()
s = 330
safe = True     # use Cholesky or LU
mu = 0          # MDI regularization parameter
its = 10
Nv = arange(250,5101,250)
Ns = len(Nv)
cTime, ncTime = zeros((Ns,)), zeros((Ns,))

# ------------------ centrosymmetric -----------------------------------

for k in range(Ns):
	N = Nv[k]
	Nh = int(N/2)
	x = linspace(-100,100,N)
	f = rand(N,1)
	startTime = time.time()
	for j in range(its):
		r = phi.distanceMatrix1d(x[:Nh],x)     # construct half-sized distance matrix
		B = phi.rbf(r,s)                       # half-sized system matrix
		ac = cn.solveCentro(B,f,mu,safe)
	cTime[k] = time.time() - startTime 
	
# ------------------ standard ------------------------------------------
	
for k in range(Ns):
	N = Nv[k]
	x = linspace(-100,100,N)
	f = rand(N,1)
	startTime = time.time()
	for j in range(its):
		r = phi.distanceMatrix1d(x)     
		B = phi.rbf(r,s)                       
		a = phi.solve(B, f, mu, safe)
	ncTime[k] = time.time() - startTime 
	
# ----------------------------------------------------------------------
	

plt.plot(Nv,cTime/ncTime,'b*') #,Nv,ones((Ns,)),'g--')
plt.xlabel('N'), plt.ylabel('execution time ratio')
plt.show()
